
import { _decorator, Component, Node, geometry, Vec3, PhysicsSystem, PhysicsRayResult, CapsuleCollider, RigidBody } from 'cc';
import { Input, KeyCode } from '../InputManager';
import { SmartFootstepSystem } from '../system/SmartFootstepSystem';
import { PersonState } from './PersonState';
const { ccclass, property } = _decorator;

@ccclass('FirstPersonController')
export class FirstPersonController extends Component {
    // [1]
    // dummy = '';

    // [2]
    // @property
    // serializableDummy = 0;

    public state:PersonState = PersonState.idle;

    @property()
    private defaultMoveSpeed:number = 1.0;

    @property()
    public runMultiplier:number = 2.0;

    @property()
    public defaultFootstepRate:number = 0.5;

	/*private normalizedmovementSpeed;
	private normalizedfootstepRate;
	private nextFootstep; */

    private footStepTime:number = 0;

    private moveSpeed:number = 1.0;

    private footstepRate:number = 0.5;

    private groundCheckDistance:number = 0.5;

    private smartFootstepSystem:SmartFootstepSystem | null = null;

    start () {
        // [3]
        this.groundCheckDistance = this.getComponent(CapsuleCollider).height / 2 + 0.25;

        //console.log("检测距离",this.groundCheckDistance);

        this.moveSpeed = this.defaultMoveSpeed;
        this.footstepRate = this.defaultFootstepRate;

        this.smartFootstepSystem = this.getComponent(SmartFootstepSystem);

    }

    update (deltaTime: number) {
         // [4]

         if(Input.getKeyDown(KeyCode.Q))
         {
            console.log("是否接触地面？",this.checkGround());
         }

         var isGround:boolean = this.checkGround();

         if(Input.getKeyDown(KeyCode.Space))
         {
             if(isGround)
             {
                this.getComponent(RigidBody)?.applyForce(new Vec3(0,480,0))
             }
            
         }
         

         if(Input.axis.x != 0 || Input.axis.y != 0)
         {
            if(Input.getKey(KeyCode.Shift))
            {
                this.state = PersonState.run;
                this.moveSpeed = this.defaultMoveSpeed * this.runMultiplier;
                this.footstepRate = this.defaultFootstepRate / this.runMultiplier;
            }else
            {
                this.state = PersonState.walk;
                this.moveSpeed = this.defaultMoveSpeed;
                this.footstepRate = this.defaultFootstepRate;
            }


            if(isGround)
            {
                this.footStepTime -= deltaTime;

                if(this.footStepTime <= 0)
                {
                    this.footStepTime = this.footstepRate;
    
                    //走了一步
                    this.smartFootstepSystem && this.smartFootstepSystem.footStep();
                }
            }

            this.node.translate(new Vec3(-Input.axis.x * (this.moveSpeed / 2) * deltaTime ,0,Input.axis.y * this.moveSpeed * deltaTime));
            

         }else
         {
            this.state = PersonState.idle;
         }

         


    }

    public checkGround():boolean
    {
        var ray:geometry.Ray = new geometry.Ray();
        ray.o = new Vec3(this.node.worldPosition.x,this.node.worldPosition.y + 0.1,this.node.worldPosition.z);
        ray.d = new Vec3(0,-1,0);

        if(PhysicsSystem.instance.raycastClosest(ray,0xffffffff,this.groundCheckDistance))
        {
            var rayResult:PhysicsRayResult = PhysicsSystem.instance.raycastClosestResult;

            if(rayResult.collider)
            {
                //console.log("rayResult.collider",rayResult.collider);
                return true;
            }
        }

        return false;
    }
}

/**
 * [1] Class member could be defined like this.
 * [2] Use `property` decorator if your want the member to be serializable.
 * [3] Your initialization goes here.
 * [4] Your update function goes here.
 *
 * Learn more about scripting: https://docs.cocos.com/creator/3.0/manual/en/scripting/
 * Learn more about CCClass: https://docs.cocos.com/creator/3.0/manual/en/scripting/ccclass.html
 * Learn more about life-cycle callbacks: https://docs.cocos.com/creator/3.0/manual/en/scripting/life-cycle-callbacks.html
 */
