
import { _decorator, Component, Node, Size, view, EventTouch, Vec2, game, Canvas, Vec3 } from 'cc';
const { ccclass, property } = _decorator;

@ccclass('GameController')
export class GameController extends Component {
    public static instance:GameController;

    @property(Node)
    public touchPlane:Node | null = null;

    @property(Canvas)
    public canvas:Canvas | null = null;

    public mouseAxis:Vec2 = new Vec2();

    private referenceWidth:number = 750; // ip6分辨率
    private referenceHeight:number = 1334 ; // ip6分辨率

    private widthRatio:number = 1;
    private heightRatio:number = 1;

    /**
     * 触摸的速度系数
     */
    public m_rateTouch:number = 960;

    public angleY:number = 0;

    public targetAngleY:number = 0;

    public angleSpeed:number = 20;

    public lerpSpeed:number = 1.5;

    public screenSize:Size = new Size();


    private isLockMouse:boolean = false;


    onLoad()
    {
        if(GameController.instance)
        {
            this.node.destroy();
            return;
        }

        GameController.instance = this;
        game.addPersistRootNode(this.node);

        this.screenSize.width = view.getVisibleSize().width;
        this.screenSize.height = view.getVisibleSize().height;


        this.widthRatio = this.referenceWidth / this.screenSize.width;
        this.heightRatio = this.referenceHeight / this.screenSize.height;

        //print("widthRatio " + widthRatio + "   heightRatio " + heightRatio);
        //print("Screen.width " + Screen.width + "   Screen.height " + Screen.height);

        //控制不同分辨率的旋转系数
        if (this.screenSize.width <= 480)
        {
            this.m_rateTouch = 960;
        }
        else if (this.screenSize.width > 1080)
        {
            this.m_rateTouch = 960;
        }

    }

    start () {

        if(this.touchPlane)
        {

            this.touchPlane.on(Node.EventType.TOUCH_START,(event:EventTouch)=>
            {
    

            },this,true);

            this.touchPlane.on(Node.EventType.MOUSE_MOVE,(event:EventTouch)=>
            {
                this.mouseAxis = event.getUIDelta();

            },this,true);

            this.touchPlane.on(Node.EventType.TOUCH_END,(event:EventTouch)=>
            {
                
            },this,true);

            this.touchPlane.on(Node.EventType.TOUCH_CANCEL,(event:EventTouch)=>
            {
      
            },this,true);

        }

        

        if(game.canvas)
        {
            //game.canvas.requestFullscreen()

            game.canvas.requestPointerLock();

            game.canvas.addEventListener('click', ()=> {
                if(document.pointerLockElement) {
                    document.exitPointerLock();
                } else {
                    game.canvas.requestPointerLock();
                }
            });

            var self = this;

            function onMouseMove(e) {
                
                //console.log("e",e.x,e.y,e.clientX,e.clientY,e.screenX,e.screenY,e.pageX,e.pageY,game.canvas.offsetLeft,game.canvas.offsetTop);

                /*let node = self.canvas.node.children[0];
                
                var pos:Vec3 = node.position;

                pos.x += e.movementX;
                pos.y -= e.movementY;*/

                self.mouseAxis.x = e.movementX;
                self.mouseAxis.y = -e.movementY;

                //node.position = pos;
            }
            // 检测鼠标锁定状态变化
            document.addEventListener('pointerlockchange', ()=> {
                //console.log('pointerlockchange: ', document.pointerLockElement)
                if (document.pointerLockElement) {
                    this.isLockMouse = true;
                    console.log("锁定鼠标");
                    game.canvas.addEventListener("mousemove", onMouseMove, false);
                } else {
                    game.canvas.removeEventListener("mousemove", onMouseMove, false);
                    console.log("解除锁定鼠标");
                }
            }, false);
        }


    }

  

    /*update (deltaTime: number) {
        

    }*/

    lateUpdate()
    {
        this.mouseAxis.x = 0;
        this.mouseAxis.y = 0;
    }

    onDestroy()
    {

    }
}

/**
 * [1] Class member could be defined like this.
 * [2] Use `property` decorator if your want the member to be serializable.
 * [3] Your initialization goes here.
 * [4] Your update function goes here.
 *
 * Learn more about scripting: https://docs.cocos.com/creator/3.0/manual/en/scripting/
 * Learn more about CCClass: https://docs.cocos.com/creator/3.0/manual/en/scripting/ccclass.html
 * Learn more about life-cycle callbacks: https://docs.cocos.com/creator/3.0/manual/en/scripting/life-cycle-callbacks.html
 */
