
import { _decorator, Component, Node, Vec3, Animation, RigidBody, geometry, PhysicsRayResult, PhysicsSystem } from 'cc';
import { Input, KeyCode } from '../InputManager';
import { SmartFootstepSystem } from '../system/SmartFootstepSystem';
import { PersonState } from './PersonState';
const { ccclass, property } = _decorator;



@ccclass('ThirdPersonController')
export class ThirdPersonController extends Component {
    // [1]
    // dummy = '';

    // [2]
    // @property
    // serializableDummy = 0;

    @property()
    public moveSpeed:number = 2.0;

    @property()
    public roatSpeed:number = 120;

    private animation:Animation | null = null;

    private _state:PersonState = PersonState.idle;

    public get state():PersonState
    {
        return this._state;
    }

    public set state(value:PersonState)
    {
        if(this._state == value)
        {
            return;
        }

        this._state = value;

        switch(this._state)
        {
            case PersonState.idle: 
                this.animation?.crossFade("Idle");
                break;
            case PersonState.walk:
                this.animation?.crossFade("Move");
                 break;
        }

    }

    private groundCheckDistance:number = 0.5;

    private footStepTime:number = 0;

    private footstepRate:number = 0.5;

    private smartFootstepSystem:SmartFootstepSystem | null = null;

    start () {
        // [3]

        this.animation = this.getComponentInChildren(Animation);
        this.smartFootstepSystem = this.getComponent(SmartFootstepSystem);
    }

    update (deltaTime: number) {
         // [4]

        var isGround:boolean = this.checkGround();

        if(Input.getKeyDown(KeyCode.Space))
        {
            if(isGround)
            {
                this.getComponent(RigidBody)?.applyForce(new Vec3(0,360,0));
            }
            
        }



        var eulerAngles:Vec3 =  this.node.eulerAngles;

        eulerAngles.y += -Input.axis.x * this.roatSpeed * deltaTime; 

        this.node.eulerAngles = eulerAngles;

        this.node.translate(new Vec3(0,0,Input.axis.y * this.moveSpeed * deltaTime));

        if(this.animation)
        {
            if(Input.axis.y != 0)
            {
                this.state = PersonState.walk;
            }else
            {
                this.state = PersonState.idle;
            }
        }

        if(Input.axis.y != 0)
        {
            if(isGround)
            {
                this.footStepTime -= deltaTime;

                if(this.footStepTime <= 0)
                {
                    this.footStepTime = this.footstepRate;

                    //走了一步
                    this.smartFootstepSystem && this.smartFootstepSystem.footStep();
                }
            }
        }

        


    }

    public checkGround():boolean
    {
        var ray:geometry.Ray = new geometry.Ray();
        ray.o = new Vec3(this.node.worldPosition.x,this.node.worldPosition.y + 0.1,this.node.worldPosition.z);
        ray.d = new Vec3(0,-1,0);

        if(PhysicsSystem.instance.raycastClosest(ray,0xffffffff,this.groundCheckDistance))
        {
            var rayResult:PhysicsRayResult = PhysicsSystem.instance.raycastClosestResult;

            if(rayResult.collider)
            {
                //console.log("rayResult.collider",rayResult.collider);
                return true;
            }
        }

        return false;
    }
}

/**
 * [1] Class member could be defined like this.
 * [2] Use `property` decorator if your want the member to be serializable.
 * [3] Your initialization goes here.
 * [4] Your update function goes here.
 *
 * Learn more about scripting: https://docs.cocos.com/creator/3.0/manual/en/scripting/
 * Learn more about CCClass: https://docs.cocos.com/creator/3.0/manual/en/scripting/ccclass.html
 * Learn more about life-cycle callbacks: https://docs.cocos.com/creator/3.0/manual/en/scripting/life-cycle-callbacks.html
 */
