
import { _decorator, Component, Node, CCString, Texture2D, AudioClip, AudioSource, Terrain, geometry, Vec3, PhysicsRayResult, PhysicsSystem, MeshRenderer } from 'cc';
import { Input, KeyCode } from '../InputManager';
const { ccclass, property } = _decorator;

@ccclass("GroundType")
export class GroundType
{
    @property({type:CCString})
    public name:string = "";

    @property({type:Texture2D})
    public texture:Texture2D = null;

    @property(AudioClip)
    public clip:AudioClip = null;
}

@ccclass('SmartFootstepSystem')
export class SmartFootstepSystem extends Component {
    @property(AudioSource)
    public footstepAudio:AudioSource = null;

    @property(GroundType)
    public groundTypes:GroundType[] = [];

    @property(Terrain)
    public terrain:Terrain = null;

    @property()
    public groundCheckDistance:number = 1;

    start () {
        // [3]

       //this.terrain.getBlock(0,0).getLayer(0).
       //this.terrain._asset

    }

    update (deltaTime: number) {
        // [4]

        if(Input.getKeyDown(KeyCode.T))
         {
            console.log("地面贴图？",this.getRendererTexture());
         }

    }

    public footStep():void
    {
        this.footstepAudio.playOneShot(this.groundTypes[0].clip);
    }

    public getRendererTexture()
    {
        var ray:geometry.Ray = new geometry.Ray();
        ray.o = new Vec3(this.node.worldPosition.x,this.node.worldPosition.y + 0.1,this.node.worldPosition.z);
        ray.d = new Vec3(0,-1,0);

        if(PhysicsSystem.instance.raycastClosest(ray,0xffffffff,this.groundCheckDistance))
        {
            var rayResult:PhysicsRayResult = PhysicsSystem.instance.raycastClosestResult;
            console.log("地面",rayResult.collider.node);

            var renderer = rayResult.collider.getComponent(MeshRenderer);

            console.log("renderer",renderer);

            if(renderer)
            {
                //console.log("rayResult.collider",rayResult.collider);
                return true;
            }
        }
    }
}

/**
 * [1] Class member could be defined like this.
 * [2] Use `property` decorator if your want the member to be serializable.
 * [3] Your initialization goes here.
 * [4] Your update function goes here.
 *
 * Learn more about scripting: https://docs.cocos.com/creator/3.0/manual/en/scripting/
 * Learn more about CCClass: https://docs.cocos.com/creator/3.0/manual/en/scripting/ccclass.html
 * Learn more about life-cycle callbacks: https://docs.cocos.com/creator/3.0/manual/en/scripting/life-cycle-callbacks.html
 */
